import platform
from wallme.managers.linux import Linux
from wallme.managers.windows import Windows
from wallme.exceptions import WallmeException

class ManagerFactory():

	def get_manager(self):
		system = platform.system()
		if(system == "Linux"):
			return Linux()
		elif(system == "Windows"):
			return Windows()
		else:
			raise WallmeException("Unknown system : '" + system + "'")


from wallme.websites import WEBSITES
from wallme.exceptions import WallmeException

class WebsiteFactory():

	def get_website(self, name):
		if(name not in WEBSITES.keys()):
			raise WallmeException("Website '" + name + "' not supported. Use the -list option to list all posibilities.")
		
		return WEBSITES[name]


from wallme import utils

NAME = 'epod'
DESCRIPTION = 'Earth Picture of the Day'
URL = 'https://epod.usra.edu/'

def pre_process():
	return None

def process(date):
	soup = utils.get_soup_from_url(URL)
	imgs = utils.find_tags_from_soup(soup, "a", attributes={"class": "asset-img-link"})
	return imgs[0].get('href')
    
def post_process(image):
	return None


import wallme.websites.websites, types

WEBSITES = {}

for item in list(globals().items()):
	if(isinstance(item, tuple)):
		if(isinstance(item[1], types.ModuleType)):
			if("websites." in item[1].__name__ and item[1].__name__ != "wallme.websites.websites"):
				WEBSITES[item[1].NAME] = item[1]


from wallme import utils

NAME = 'national-geographic'
DESCRIPTION = 'One photo of the world every day'
URL = 'https://www.nationalgeographic.com/photography/photo-of-the-day'

def pre_process():
	return None

def process(date):
	json = utils.get_json_from_url('https://www.nationalgeographic.com/photography/photo-of-the-day/_jcr_content/.syndication-gallery.json')
	renditions = json['items'][0]['image']['renditions']
	sorted_renditions = sorted(renditions, key=lambda i:int(i['width']), reverse=True)
	return sorted_renditions[0]['uri']

def post_process(image):
	return None


import os
from wallme.managers.manager import Manager
from wallme.exceptions import WallmeException

class Linux(Manager):
        
	SERVICE_FILE = "/lib/systemd/system/wallme.service"

	def set(self, website, test=False):
		super().download(website)
		if(not test):
			if os.system("/usr/bin/gsettings set org.gnome.desktop.background picture-uri " + self.IMAGE) != 0:
				raise WallmeException("Cannot set wallpaper")
            
	def set_startup(self, website):
		
		self.unset_startup()
        
		user = os.popen("who | awk 'FNR == 1 {print $1}'").read().rstrip()
		
		if(not os.path.exists(self.SERVICE_FILE)):
			with open(self.SERVICE_FILE, "wt") as f:
				f.write("[Unit]\n\
Description=Wallme service.\n\
After=network-online.target\n\
\n\
[Service]\n\
Type=simple\n\
User=" + user + "\n\
Group=" + user + "\n\
ExecStart=/usr/local/bin/wallme -set " + website.NAME + "\n\
\n\
[Install]\n\
WantedBy=multi-user.target")

		os.system("sudo systemctl enable wallme")

	def unset_startup(self):
		if(os.path.exists(self.SERVICE_FILE)):
			os.remove(self.SERVICE_FILE)
			os.system("sudo systemctl disable wallme")


import ctypes, os
from wallme.managers.manager import Manager
from wallme.exceptions import WallmeException

class Windows(Manager):

	STARTUP_FOLDER = os.path.expandvars(r'%APPDATA%\Microsoft\Windows\Start Menu\Programs\Startup')
	STARTUP_FILE = STARTUP_FOLDER + "\\wallme.bat"

	def set(self, website, test=False):
		super().download(website)
		if(not test):
			if ctypes.windll.user32.SystemParametersInfoW(20, 0, self.IMAGE, 3) != 1:
				raise WallmeException("Cannot set wallpaper")

	def set_startup(self, website):
		if(not os.path.exists(self.STARTUP_FOLDER)):
			os.makedirs(STARTUP_FOLDER)
		
		self.unset_startup()
        
		if(not os.path.exists(self.STARTUP_FILE)):
			with open(self.STARTUP_FILE, "wt") as f:
				f.write("wallme -set " + website.NAME)

	def unset_startup(self):
		if(os.path.exists(self.STARTUP_FILE)):
			os.remove(self.STARTUP_FILE)


import wallme.websites.apod
import wallme.websites.astrobin
import wallme.websites.bing
import wallme.websites.epod
import wallme.websites.nasa
import wallme.websites.nationalgeographic
import wallme.websites.tetras
import wallme.websites.theguardian
import wallme.websites.thetelegraph
import wallme.websites.wikipedia



import requests
from datetime import date
from pathlib import Path
import webbrowser

class Manager():

	IMAGE = str(Path.home()) + '/wallme.jpg'

	def download(self, website):
		#Get image url
		pre_process_result = website.pre_process()
        
		#Get image url
		image_url = website.process(date.today())

		#Download image
		img_data = requests.get(image_url).content
		with open(self.IMAGE, 'wb') as handler:
			handler.write(img_data)

        #Flip image if necessary
		post_process_result = website.post_process(self.IMAGE)

	def info(self, website):
		webbrowser.open(website.URL, new=2)


import sys
if sys.version_info < (3, 5):
    sys.exit('Y2T requires Python 3.5+')

long_description = open("README.md", "r").read()

import setuptools
setuptools.setup(
	name='wallme',
	version='1.3.1',
	author='LucBerge',
	author_email='lucas.bergeron@outlook.fr',
	description="Change your wallpaper every day",
	long_description=long_description,
	long_description_content_type="text/markdown",
	url='https://github.com/LucBerge/wallme',
	packages=setuptools.find_packages(),
	install_requires=['requests', 'bs4', 'Pillow'],
	entry_points = {
        'console_scripts': ['wallme=wallme.main:main'],
    }
)



import requests, json
from bs4 import BeautifulSoup

def get_webpage_from_url(url):
	webpage = requests.get(url)
	if webpage.status_code != 200:
		raise Exception("Cannot retrieve webpage '" + url + "' (error " + str(webpage.status_code) + "). Please make sure the url is valid.")
	return webpage.text

def get_json_from_url(url):
	return json.loads(get_webpage_from_url(url))
	
def get_soup_from_url(url, parser='html.parser'):
	return BeautifulSoup(get_webpage_from_url(url), parser)

def find_tags_from_soup(soup, tag, attributes=None):
	if(attributes):
		tags = soup.findAll(tag, attributes)
	else:
		tags = soup.find_all(tag)
 
	if(len(tags) == 0):
		raise Exception("Couldn't find tag '" + tag + "' with attributes '" + str(attributes) + "'.")
	return tags